/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LOLfox.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ["LOLfox"];
const Cc = Components.classes;
const Ci = Components.interfaces;

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://lolfox-modules/util.js");


function onChromeLoad(evt) {
  util.log("onChromeLoad");

  var win = evt.currentTarget;
  win.removeEventListener("load", onChromeLoad, false);
  win.addEventListener("unload", onChromeUnload, false);

  if (Prefs.isEnabled() === false) {
    return;
  }

  var engine = getEngine();

  if (engine.isActive()) {
    engine.enableWindow(win);
  } else {
    PrefCommands.initEngine();
  }

}


function onChromeUnload(evt) {
  util.log("onChromeUnload");
  var win = evt.currentTarget;
  win.removeEventListener("unload", onChromeUnload, false);

  if (Prefs.isEnabled()) {
    getEngine().disableWindow(win);
  }
}




var m_engineLoaded = false;

function getEngine() {
  m_engineLoaded = true;
  Components.utils.import("resource://lolfox-modules/engine.js");
  return Engine;
}


const LOLfox = {

  initChrome: function(win) {
    win.addEventListener("load", onChromeLoad, false);
  },

  updateMenu: function(popup) {
    var content = Prefs.getBoolPref("content.enabled");

    popup.querySelector("menuitem[name=lolfox]")
         .setAttribute("checked", content);
  },

  toggleApp: function(menuitem) {
    var newValue = menuitem.getAttribute("checked") === "true";
    switch (menuitem.getAttribute("name")) {
      case "lolfox":
        Prefs.setBoolPref("content.enabled", newValue);
        break;
      default:
        throw "toggleApp";
    }
  },


  setLang: function(menuitem) {
    var currentMode = Prefs.language();
    util.log("setLang=" + currentMode);
    var newMode = menuitem.value;
    if (newMode !== currentMode) {
      Prefs.setStringPref("lang", newMode);
    }
  }

};

    this._prefs = Cc["@mozilla.org/preferences-service;1"].getService(Ci.nsIPrefBranch2);

const Prefs = {
  _prefBranch: null,

  register: function() {
    this._prefBranch = Cc["@mozilla.org/preferences-service;1"]
                        .getService(Ci.nsIPrefService)
                        .getBranch("extensions.lolfox@hultmann.")
                        .QueryInterface(Ci.nsIPrefBranch2);
    this._prefBranch.addObserver("", this, false);
  },

  //this.unregister = function() {
  //  this._prefBranch.removeObserver("", this);
  //};

  QueryInterface: XPCOMUtils.generateQI([Ci.nsIObserver]),

  observe: function(subject, topic, data) {
    util.log("Prefs.observe " + data);

    if (topic !== "nsPref:changed") {
      return;
    }

    if (data in PrefCommands) {
      PrefCommands[data]();
    }
  },

  getStringPref: function(name) {
    try {
      var value = this._prefBranch.getComplexValue(name, Ci.nsISupportsString);
      return value.data;
    } catch (ex) {
      Components.utils.reportError(ex);
      return null;
    }
  },

  getBoolPref: function(name) {
    return this._prefBranch.getBoolPref(name);
  },

  setBoolPref: function(name, v) {
    return this._prefBranch.setBoolPref(name, v);
  },

  setStringPref: function(name, value) {
    var i = Ci.nsISupportsString;
    try {
      var w = Cc["@mozilla.org/supports-string;1"].createInstance(i);
      w.data = value;
      this._prefBranch.setComplexValue(name, i, w);
    } catch (ex) {
      Components.utils.reportError(ex);
      throw "setStringPref " + ex;
    }
  },

  language: function() {
    var currentMode = this.getStringPref("lang");
    switch (currentMode) {
      case "lolcat":
        // allowed values
        return currentMode;
      default:
        // invalid
        return "icq";
    }
  },

  isEnabled: function() {
    return this.getBoolPref("chrome.enabled") ||
           this.getBoolPref("content.enabled");
  }

};

Prefs.register();
//Prefs.unregister();


const PrefCommands = {

  initEngine: function() {
    if (!Prefs.isEnabled()) {
      throw "initEngine"
    }
    getEngine().start(Prefs.language(),
                      Prefs.getBoolPref("chrome.enabled"),
                      Prefs.getBoolPref("content.enabled"));
  },

  "lang": function() {
    util.log("PrefCommands lang");
    this.initEngine();
  },

  "content.enabled": function() {
    util.log("PrefCommands content.enabled");

    if (Prefs.isEnabled()) {
      this.initEngine();

    } else if (m_engineLoaded) {
      getEngine().stop();
    }
  },

  "chrome.enabled": function() {
    util.log("PrefCommands chrome.enabled");
    if (Prefs.getBoolPref("chrome.enabled")) {
      //getEngine().updateWindowsChrome();
    }
  }

};
