/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LOLfox.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */

const EXPORTED_SYMBOLS = ["Engine"];

Components.utils.import("resource://gre/modules/XPCOMUtils.jsm");
Components.utils.import("resource://lolfox-modules/util.js");

const Cc = Components.classes;
const Ci = Components.interfaces;

util.log("load engine.js");

const Engine = {

  translator: null,

  isActive: function() {
    return this.translator !== null;
  },

  start: function(idLang, chrome, content) {
    util.log("Engine.start");
    this.stop();

    this.translator = getTranslator(idLang);
    if (this.translator === null) {
      util.log("@start this.translator === null");
      throw "this.translator === null";
    }
    this.translator.init();
    var winEnum = util.browserWindowsEnum();
    while (winEnum.hasMoreElements()) {
      this.enableWindow(winEnum.getNext());
    }

  },

  stop: function() {
    if (this.translator === null) {
      return;
    }

    util.log("Engine.stop");


    var winEnum = util.browserWindowsEnum();
    while (winEnum.hasMoreElements()) {
      Engine.disableWindow(winEnum.getNext());
    }

    this.translator.dispose();
    this.translator = null;
  },


  enableWindow: function(win) {
    util.log("Engine.enableWindow");
    var g = win.getBrowser();
    g.addProgressListener(domPollingListeners, Ci.nsIWebProgress.NOTIFY_LOCATION);

    g.addEventListener("DOMContentLoaded", onDomLoaded, false);
    g.addEventListener("DOMTitleChanged", onTitle, false);
    g.addEventListener("pageshow", onPageshow, false);
    //g.addEventListener("pagehide", onPagehide, false); //pagehide doesn't fire when a tab closes

  },

  disableWindow: function(win) {
    util.log("Engine.disableWindow");
    var g = win.getBrowser();

    g.removeProgressListener(domPollingListeners);
    g.removeEventListener("DOMContentLoaded", onDomLoaded, false);
    g.removeEventListener("DOMTitleChanged", onTitle, false);
    g.removeEventListener("pageshow", onPageshow, false);
    //g.removeEventListener("pagehide", onPagehide, false);

  }

};


function getTranslator(idLang) {
  switch (idLang) {
    case "lolcat":
      Components.utils.import("resource://lolfox-modules/lolcat.js");
      break;
  }
  if (Translator === null) {
    util.log("getTranslator Translator === null");
    throw "Translator === null " + idLang;
  }

  return Translator;
};




function onDomLoaded(evt) {
  util.log("onDomLoaded " + evt.target.location.href);
  loaded(evt.target.defaultView);
}

function onPageshow(evt) {
  util.log("onPageshow " + evt.target.location.href);
  loaded(evt.target.defaultView);
}

function loaded(domWin) {
  if (NodeStore.getValue(domWin.document, "dom-loaded") !== null) { // onDomLoaded or onPageshow
    return;
  }
  util.log("loaded " + domWin.document.location.href);
  NodeStore.setValue(domWin.document, "dom-loaded", "yep");
  var doc = domWin.document;
  Parse.completeNode(doc.documentElement);
  addDocMutationListeners(doc);

}


function onTitle(evt) {
  var keyAntiLoopText = "previousTranslatedText";
  var doc = evt.target;

  var node = doc.querySelector("title");
  if (!node) {
    util.log("no title tag? " + doc.location.href + doc.contentType);
    return;
  }

  var currentText = doc.title;
  var previousText = NodeStore.getValue(node, keyAntiLoopText);

  if (previousText !== null) {
    if (currentText === previousText) {
      NodeStore.setValue(node, keyAntiLoopText, null);
      // DOMTitleChanged triggered by handler! doc.title = txt
      return;
    }
  }

  var newTranslation = Engine.translator.translate(currentText);
  NodeStore.setValue(node, keyAntiLoopText, newTranslation);
  doc.title = newTranslation;
}


const NodeStore = {

  getValue: function(node, name) {
    if ("lolfox" in node) {
      var obj = node["lolfox"];
      if (name in obj) {
        return obj[name];
      }
    }
    return null;
  },

  setValue: function (node, name, value) {
    var obj = this._getStore(node)
    if (value === null) {
      delete obj[name];
    } else {
      obj[name] = value;
    }
  },

  _getStore: function(node) {
    if ("lolfox" in node) {
      return node["lolfox"];
    }
    var obj = [];
    node["lolfox"] = obj;
    return obj;
  }

};

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LOLfox.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */




const domPollingListeners = {
  QueryInterface: XPCOMUtils.generateQI([Ci.nsIWebProgressListener, Ci.nsISupportsWeakReference]),
  onStateChange: function() {},
  onSecurityChange: function() {},
  onProgressChange: function() {},
  onStatusChange: function() {},
  onLinkIconAvailable: function() {},
  onLocationChange: function(aWebProgress, aRequest, aLocation) {
    if (aWebProgress.isLoadingDocument && aWebProgress.DOMWindow) {
      tryPolling(aWebProgress.DOMWindow);
    }
  }
};


// prevent win.domPolling_mgxtr = 0;
//         win.domPolling_mgxtr = '';
//         win.domPolling_mgxtr = null;
//         win.domPolling_mgxtr = false;
function hasPolling(domWin) {
  //return ("domPolling_mgxtr" in domWin) && (domWin.domPolling_mgxtr !== null);
  return NodeStore.getValue(domWin.document, "domPolling") !== null;
}


function pollingFrames(myWin) {
  for (var idx = 0, len = myWin.frames.length; idx < len; idx++) {
    var domWin = myWin.frames[idx].window;
    tryPolling(domWin);
    pollingFrames(domWin);
  }
}


function tryPolling(domWin) {
  if (hasPolling(domWin)) {
    return;
  }

  var doc = domWin.document;
  if (doc.location.href === "about:blank") {
    return;
  }


  util.log("tryPolling OK " + doc.location.href);
  NodeStore.setValue(doc, "domPolling", new DomPolling(domWin));
}


function DomPolling(domWin) {
  this._start = new Date().getTime();
  this._domWin = domWin;
  domWin.setTimeout(parseWindow, 0, this);
}



function pollingDispose(win) {
  util.log("pollingDispose " + win.document.location);
  if (hasPolling(win)) {
    NodeStore.setValue(win.document, "domPolling", null);
  } else {
    // frame pode dom sem que polling ja tenha verificado a existência do frame
    util.log("*domPolling=null");
  }
}


function parseWindow(pollingData) {
  var win = pollingData._domWin;
  util.log("parseWindow " + win.document.location);

  var currentWin = Cc["@mozilla.org/appshell/window-mediator;1"]
                    .getService(Ci.nsIWindowMediator)
                    .getMostRecentWindow("navigator:browser")
                    .getBrowser()
                    .selectedBrowser
                    .contentDocument
                    .defaultView;

  if (win.top !== currentWin) {
    // polling apenas na aba ativa
    util.log("pollingDispose! win.top !== currentWin");
    win.setTimeout(pollingDispose, 0, win); // 0, pois user poderia voltar a aba antes de pollingDispose ser chamado
    return;
  }

  pollingFrames(win);

  if (NodeStore.getValue(win.document, "dom-loaded") !== null) {
    win.setTimeout(pollingDispose, 999, win); // sem pressa
    return;
  }

  // XULDocument pode nao ter document.documentElement durante o carregamento!
  Parse.loadingNode(win.document.documentElement);

  if ((new Date().getTime() - pollingData._start) < 30000) { // evita polling em conexões infinitas
    win.setTimeout(parseWindow, 0, pollingData);
  } else {
    util.log("DomPolling STOP (timeout)");
  }

}
/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LOLfox.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */



function addDocMutationListeners(doc) {
  util.log("addDocMutationListeners " + doc.location.href);
  doc.addEventListener("DOMNodeInserted", onNodeInserted, false);
  doc.addEventListener("DOMAttrModified", onAttributeModified, false);
  //doc.addEventListener("DOMCharacterDataModified", onDataModified, false);
  doc.defaultView.addEventListener("pagehide", onPagehide, false);
};


function removeDocMutationListeners(doc) {
  util.log("removeDocMutationListeners");
  doc.removeEventListener("DOMNodeInserted", onNodeInserted, false);
  doc.removeEventListener("DOMAttrModified", onAttributeModified, false);
  //doc.removeEventListener("DOMCharacterDataModified", onDataModified, false);
  doc.defaultView.removeEventListener("pagehide", onPagehide, false);
};


function onPagehide(ev) {
  var doc = ev.target;
  util.log("onPagehide " + doc.location.href);
  removeDocMutationListeners(doc);
}


function onDataModified(evt) {
  util.log('onDataModified ' + ' ' + evt.target + ' data=' + evt.target.data + ' prev=' + evt.prevValue + ' new=' + evt.newValue + '\nhtml=' + evt.target.innerHTML + ',' + evt.target.originalText_mgxtr);
  Parse.completeNode(evt.target);
}


function onNodeInserted(evt) {
  if (evt.target.nodeType === TEXT_NODE) {
    var n = evt.target.parentNode.nodeName;
    if (n === "title" || n === "TITLE") {
      util.log('onNodeInserted title='+evt.target.textContent);
      return;
    }
  }
  Parse.completeNode(evt.target);
}


function onAttributeModified(evt) {
  switch (evt.attrChange) {
    case 1: // MODIFICATION
    case 2: // ADDITION
      break;
    default:
      return;
  }

  switch (evt.attrName) {
    case "id":
    case "class":
    case "style":
      return;
  }

  if (evt.attrName.indexOf("on") === 0) {
    // onclick etc
    return;
  }

  Parse.attr(evt.target);
}

/* ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 *
 * The contents of this file are subject to the Mozilla Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and limitations under the
 * License.
 *
 * The Original Code is LOLfox.
 *
 * The Initial Developer of the Original Code is
 * Jeferson Hultmann <hultmann@gmail.com>
 * Portions created by the Initial Developer are Copyright (C) 2009
 * the Initial Developer. All Rights Reserved.
 *
 * Contributor(s):
 *
 * Alternatively, the contents of this file may be used under the terms of
 * either the GNU General Public License Version 2 or later (the "GPL"), or
 * the GNU Lesser General Public License Version 2.1 or later (the "LGPL"),
 * in which case the provisions of the GPL or the LGPL are applicable instead
 * of those above. If you wish to allow use of your version of this file only
 * under the terms of either the GPL or the LGPL, and not to allow others to
 * use your version of this file under the terms of the MPL, indicate your
 * decision by deleting the provisions above and replace them with the notice
 * and other provisions required by the GPL or the LGPL. If you do not delete
 * the provisions above, a recipient may use your version of this file under
 * the terms of any one of the MPL, the GPL or the LGPL.
 *
 * ***** END LICENSE BLOCK ***** */


const TEXT_NODE = 3;
const ELEMENT_NODE = 1;


const Parse = {

  completeNode: function(myNode) {
    this._nodeElementRecursive(myNode, false);
  },

  loadingNode: function(myNode) {
    this._nodeElementRecursive(myNode, true);
  },

  attr: function(myNode) {
    if (myNode.hasAttribute("title")) {
      var t = myNode.getAttribute("title");


      var ok = true;
      var prevTranslation = NodeStore.getValue(myNode, "translated_attr_title");
      if (prevTranslation !== null) {
        if (t === prevTranslation) {
          //util.log("attr title loop blocked! " + myNode.ownerDocument.contentType + '\b'+t);
          ok = false;
        }
      }

      if (ok) {
        var newTranslation = Engine.translator.translate(t);
        NodeStore.setValue(myNode, "translated_attr_title", newTranslation);
        myNode.setAttribute("title", newTranslation);
        //util.log("new title \n" + t +'\n' +prevTranslation);
      }




    }
    switch (myNode.nodeName) {
      case "IMG":
      case "img":
        var t = myNode.alt;
        if (t.length > 0) {


          var ok = true;
          var prevTranslation = NodeStore.getValue(myNode, "translated_attr_alt");
          if (prevTranslation !== null) {
            if (t === prevTranslation) {
              util.log("attr alt loop blocked!");
              ok = false;
            }
          }

          if (ok) {
            var newTranslation = Engine.translator.translate(t);
            NodeStore.setValue(myNode, "translated_attr_alt", newTranslation);
            myNode.alt = newTranslation;
          }


        }
        break;

      case "INPUT":
      case "input":
        // mudar submit sem "name" não deve causar efeito colateral
        if (myNode.name.length === 0 && myNode.type === "submit") {

          var t = myNode.value;

          var ok = true;
          var prevTranslation = NodeStore.getValue(myNode, "translated_attr_value");
          if (prevTranslation !== null) {
            if (t === prevTranslation) {
              util.log("attr value loop blocked!");
              ok = false;
            }
          }

          if (ok) {
            var newTranslation = Engine.translator.translate(t);
            NodeStore.setValue(myNode, "translated_attr_value", newTranslation);
            myNode.value = Engine.translator.translate(newTranslation);
          }

        }
        break;
    }
  },

  _nodeElementRecursive: function(myNode, nodeIsLoading) {
    //util.log('_nodeElementRecursive ' + myNode + ',' + myNode.textContent);

    switch (myNode.nodeType) {
      case TEXT_NODE:
        if (nodeIsLoading) {
          break;
        }

        // se myNode=='HTML',parentNode==document
        switch (myNode.parentNode.nodeName) {
          case "STYLE":
          case "SCRIPT":
          case "TITLE":
          case "style": // xml==>lowercase
          case "script":
          case "title":
            break;

          default:
            /*
            if ('originalText_mgxtr' in myNode) { //&& (myNode.originalText_mgxtr === null)) {
              util.log('NODE TEXT skip '+ myNode.originalText_mgxtr + '\n' + myNode.data + ' '  + myNode.ownerDocument.location.href + myNode.originalText_mgxtr);
              break;
            }
            myNode.originalText_mgxtr = null; // parsed flag
            */

            const txt = myNode.data;
            if (txt.length > 0 && (this._isWhitespace(txt) === false)) {
              myNode.data = Engine.translator.translate(txt);
            }
            break;
        }
        break;

      case ELEMENT_NODE:
        //} else if ('childNodes' in myNode) {
        const children = myNode.childNodes;
        const len = children.length;
        /*
        var alreadyParsed;
        if ("childrenFullyParsed_mgxtr" in myNode) {
          alreadyParsed = NodeStore.getValue(myNode, "childrenFullyParsed");
        } else {
          alreadyParsed = 0;
        }*/

        var alreadyParsed = NodeStore.getValue(myNode, "childrenFullyParsed");
        if (alreadyParsed === null) {
          alreadyParsed = 0;
        }


        // >>> 0 html                  0
        //        0 head               x
        // >>>    1 body               3
        //            0 div            x
        //            1 #text
        //            2 div            0
        //                 0 p         0
        //                 1 #text
        // >>>        3 div            2
        //                 0 #text
        //                 1 p         x
        // >>>             2 p         x
        for (var idx = alreadyParsed; idx < len; idx++) {
          var childIsLoading = nodeIsLoading && (idx === (len - 1));
          this._nodeElementRecursive(children[idx], childIsLoading);
        }

        if (nodeIsLoading) {
          // incremental translation
          //             myNode.childrenFullyParsed_mgxtr = (len === 0) ? 0 : len - 1;
          NodeStore.setValue(myNode, "childrenFullyParsed", (len === 0) ? 0 : len - 1);
        } else {
          //             myNode.childrenFullyParsed_mgxtr = len;
          NodeStore.setValue(myNode, "childrenFullyParsed", len);
          this.attr(myNode);
        }
        break;
    }
  },


  _isWhitespace: function(txt) {
    return !(/[^\t\n\r ]/.test(txt));
  }

};
